/*
   We can combine the two patterns for creating a thread
   and give a Thread object three nontrivial run() methods!
*/

public class Threads_11_experiment_2
{
   public static void main(String[] args)
   {
      final Runnable f = new MyFunction();

      final Thread t = new MyThread(f); // This Thread object has 3 run() methods.

      t.start(); // Which run() method gets called?

      try
      {
         t.join();
      }
      catch (InterruptedException e){e.getMessage();}

      System.out.printf("All done (tid = %d).\n", Thread.currentThread().getId());
      return;
   }
}


class MyThread extends Thread
{
   // Explicitly give this class two constructors.
   public MyThread()
   {
      super();
   }
   public MyThread(Runnable task)
   {
      super(task);
   }

   @Override
   public void run() // run() method #1, runs first. Override run() from the Thread super-class.
   {
      System.out.printf("This is run() in class MyThread extends Thread (tid = %d).\n",
                        this.getId());

      super.run(); // run() method #2, runs second. Call run() from the Thread super-class,
   }               // which will call the Runnable's run() method.
}


class MyFunction implements Runnable
{
   @Override
   public void run() // run() method #3, runs third.
   {
      System.out.printf("This is run() in class MyFunction implements Runnable (tid = %d).\n",
                         Thread.currentThread().getId());
   }
}

// https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/Thread.html#start()
// https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/Thread.html#run()
